#include <stdio.h>
#include <ldap.h>

#define SEARCHBASE	"o=Ace Industry, c=US"
#define HOST			"ldap.netscape.com"

#ifdef LDAP_OPT_SIZELIMIT		/* only defined in Netscape SDK */
#define GET_LDERROR( ld )		ldap_get_lderrno ( (ld), NULL, NULL )
#else										/* UMICH SDK */
#define GET_LDERROR( ld )		(ld)->ld_errno
#endif

void
report_ldap_error( char *s, int err )
{
	char	*msg;

	msg = ldap_err2string( err );
	fprintf( stderr, "An LDAP error occurred while %s\n(%s)\n", s, msg );
}

main()
{
	LDAP		*ld;
	LDAPMessage	*result, *e;
	char		*dn;
	int		rc, msgid, lderr;

	if ( (ld = ldap_init( HOST, LDAP_PORT ))
	    == NULL ) {
		perror( HOST );
		exit( 1 );
	}

	if ( ldap_simple_bind_s( ld, NULL, NULL ) != LDAP_SUCCESS ) {
		report_ldap_error( "binding", GET_LDERROR( ld ));
		ldap_unbind( ld );
		exit( 1 );
	}

	if ( (msgid = ldap_search( ld, SEARCHBASE, LDAP_SCOPE_SUBTREE,
	    "(sn=Jensen)", NULL, 0 )) == -1 ) {
		report_ldap_error( "initiating search", GET_LDERROR( ld ));
		ldap_unbind( ld );
		exit( 1 );
	}

	while ( (rc = ldap_result( ld, msgid, 0, NULL, &result ))
	    == LDAP_RES_SEARCH_ENTRY ) {
		if ( (e = ldap_first_entry( ld, result )) == NULL ) {
			report_ldap_error( "getting entry",
			    GET_LDERROR( ld ));
		} else if (( dn = ldap_get_dn( ld, e )) == NULL ) {
			report_ldap_error( "getting DN", GET_LDERROR( ld ));
		} else {
			printf( "Found: %s\n", dn );
			ldap_memfree( dn );
		}
		ldap_msgfree( result );
	}

	if ( rc == LDAP_RES_SEARCH_RESULT ) {
		lderr = ldap_result2error( ld, result, 1 );
		if ( lderr != LDAP_SUCCESS ) {
			report_ldap_error( "searching", lderr );
		}
	} else {
		report_ldap_error( "waiting for result", GET_LDERROR( ld ));
	}

	ldap_unbind( ld );
	exit( 0 );
}
