#include <stdio.h>
#include <ldap.h>

#define SEARCHBASE	"o=Netscape Communications Corp., c=US"

main()
{
	LDAP		*ld;
	LDAPMessage	*result;
	int		nentries;

	if ( (ld = ldap_init( "ggood", LDAP_PORT ))
	   == NULL) {
		perror( "ldap_init" );
		exit( 1 );
	}

	if ( ldap_simple_bind_s( ld, NULL, NULL ) != LDAP_SUCCESS ) {
		ldap_perror( ld, "ldap_simple_bind_s" );
		exit( 1 );
	}

	if ( ldap_search_s( ld, SEARCHBASE, LDAP_SCOPE_SUBTREE,
	   "(sn=Jensen)", NULL, 0, &result ) != LDAP_SUCCESS ) {
		ldap_perror( ld, "ldap_search_s" );
		exit( 1 );
	}

	if ( (nentries = ldap_count_entries( ld, result )) == -1 ) {
		ldap_perror( ld, "ldap_count_entries" );
		exit( 1 );
	}

	printf( "Found %d Jensens at Ace Industry\n", nentries );
	ldap_msgfree( result );
	ldap_unbind( ld );
	return( 0 );
}

void
print_entry_dns( LDAP *ld, LDAPMessage *result ) 
{
	LDAPMessage	*e;
	char		*dn; 

	for ( e = ldap_first_entry( ld, result ); e != NULL;
	   e = ldap_next_entry(ld, e ) ) {
		if ( (dn = ldap_get_dn( ld, e )) != NULL ) {
			printf( "dn: %s\n", dn );
			ldap_memfree( dn );
		} else {
			ldap_perror( ld, "ldap_get_dn" );
		}
	}
}

int
ldap_count_entries( LDAP *ld, LDAPMessage *result ) 
{
	LDAPMessage	*e;
	int		nentries = 0; 

	for ( e = ldap_first_entry( ld, result ); e != NULL;
	   e = ldap_next_entry(ld, e ) ) {
		nentries++;
	}
	return( nentries );
}

void
print_entry_attr_names( LDAP *ld, LDAPMessage *result ) 
{
	LDAPMessage	*e;
	BerElement	*ber;
	char		*dn, *attr; 

	for ( e = ldap_first_entry( ld, result ); e != NULL;
	   e = ldap_next_entry(ld, e ) ) {
		if ( (dn = ldap_get_dn( ld, e )) != NULL ) {
			printf( "dn: %s\n", dn );
			ldap_memfree( dn );
		}
		for ( attr = ldap_first_attribute( ld, e, &ber );
		   attr != NULL;
		   attr = ldap_next_attribute( ld, e, ber ) ) {
			printf( "attr: %s\n", attr );
			ldap_memfree( attr );
		}
		if ( ber != NULL ) {
			ber_free( ber, 0 );
		}
	}
}

void
print_entries_with_values( LDAP *ld, LDAPMessage *result ) 
{
	LDAPMessage	*e;
	BerElement	*ber;
	char		*dn, *attr;
	char		**vals;
	int		i;

	for ( e = ldap_first_entry( ld, result ); e != NULL;
	   e = ldap_next_entry(ld, e ) ) {
		if ( (dn = ldap_get_dn( ld, e )) != NULL ) {
			printf( "dn: %s\n", dn );
			ldap_memfree( dn );
		}
		for ( attr = ldap_first_attribute( ld, e, &ber );
		   attr != NULL;
		   attr = ldap_next_attribute( ld, e, ber ) ) {
			if ( (vals = ldap_get_values( ld, e, attr ))
			   != NULL ) {
				for ( i = 0; vals[i] != NULL; i++ ) {
					printf( "%s: %s\n", attr, vals[i] );
				}
				ldap_value_free( vals );
			}
			ldap_memfree( attr );
		}
		printf( "\n" );
		if ( ber != NULL ) {
			ber_free( ber, 0 );
		}
	}
}

