#ifdef TEST
#include <stdio.h>
#include <ldap.h>

void print_values( char *label, char **vals );
#endif /* TEST */


void
print_entry( LDAP *ld, LDAPMessage *entry )
{
	char	**vals;

	if (( vals = ldap_get_values( ld, entry, "sn" )) != NULL ) {
		print_values( "Surname", vals );
		ldap_value_free( vals );
	}

	if (( vals = ldap_get_values( ld, entry, "cn" )) != NULL ) {
		(void)ldap_sort_values( ld, vals, ldap_sort_strcasecmp );
		print_values( "Common Name", vals );
		ldap_value_free( vals );
	}

	if (( vals = ldap_get_values( ld, entry, "drink" )) != NULL ) {
		(void)ldap_sort_values( ld, vals, ldap_sort_strcasecmp );
		print_values( "Favorite Drink", vals );
		ldap_value_free( vals );
	}
}


void
print_values( char *label, char **vals )
{
	int		i;

	puts( label );
	for ( i = 0; vals[ i ] != NULL; ++i ) {
		printf( "\t%s\n", vals[ i ] );
	}
}


#ifdef TEST

#define HOST "belltower"

main( int argc, char **argv )
{
	int		i, rc;
	LDAP		*ld;

	if ( argc < 2 ) {
		fprintf( stderr, "%s: DN...\n", argv[ 0 ] );
		exit( 1 );
	}

	if ( (ld = ldap_init( HOST, LDAP_PORT )) == NULL ) {
		perror( HOST );
		exit( 1 );
	}

	for ( i = 1; i < argc; ++i ) {
		read_and_print( ld, argv[ i ] );
	}

	exit( 0 );
}


read_and_print( LDAP *ld, char *dn )
{
	LDAPMessage	*res;

	if ( ldap_search_s( ld, dn, LDAP_SCOPE_BASE, "(objectClass=*)",
	    NULL, 0, &res ) == LDAP_SUCCESS ) {
		print_entry( ld, ldap_first_entry( ld, res ));
		ldap_msgfree( res );
		putchar( '\n' );
	}
}
#endif /* TEST */
