#include <stdio.h>
#include <ldap.h>
#include <time.h>

#define BINDDN		"cn=Directory Manager, o=Ace industry, c=US"
#define BINDPWD		"ldap=fun"
#define DELETEDN	"cn=Sam Carter, ou=Accounting, o=Ace Industry, c=US"
#define TIMELIMIT	30	/* seconds per operation */

int
wait4result( LDAP *ld, char *msg, int limit, int msgid )
{
	int			rc;
	struct timeval		tv;
	LDAPMessage		*result;

	tv.tv_sec = limit;
	tv.tv_usec = 0;

	rc = ldap_result( ld, msgid, 0, &tv, &result );

	if ( rc == -1 ) {
		ldap_perror( ld, msg );
		return( -1 );
	} else if ( rc == 0 ) {
		fprintf( stderr, "The %s took too long!\n", msg );
		ldap_abandon( ld, msgid );
		return( -1 );
	}

	if ( ldap_result2error( ld, result, 1 ) != LDAP_SUCCESS ) {
		ldap_perror( ld, msg );
		return( -1 );
	}

	return( 0 );
}


main()
{
	LDAP	*ld;
	int	msgid;

	/* get an LDAP session handle */
	if ( (ld = ldap_init( "ldap.netscape.com", LDAP_PORT ))
	    == NULL ) {
		perror( "ldap_init" );
		exit( 1 );
	}

	/* send the bind request */
	if ( (msgid = ldap_simple_bind( ld, BINDDN, BINDPWD )) == -1 ) {
		ldap_perror( ld, "LDAP bind" );
		ldap_unbind( ld );
		exit( 1 );
	}

	/* wait up to TIMELIMIT seconds for the bind to complete */
	if ( wait4result( ld, "LDAP bind", TIMELIMIT, msgid ) != 0 ) {
		ldap_unbind( ld );
		exit( 1 );
	}

	/* send the delete request */
	if ( (msgid = ldap_delete( ld, DELETEDN )) == -1 ) {
		ldap_perror( ld, "LDAP delete" );
		ldap_unbind( ld );
		exit( 1 );
	}

	/* wait up to TIMELIMIT seconds for the delete to complete */
	if ( wait4result( ld, "LDAP delete", TIMELIMIT, msgid ) != 0 ) {
		ldap_unbind( ld );
		exit( 1 );
	}

	ldap_unbind( ld );
	exit( 0 );
}
