#ifdef TEST
#include <stdio.h>
#include <string.h>
#endif



#include <ldap.h>

char	*binddn;
char	*bindpasswd;

int
get_rebind_credentials( LDAP *ld, char **dnp, char **pwp,
    int *authmethodp, int freeit, void *arg )
{
	if ( freeit == 0 ) {
	    /* first time called: return a copy of the DN and password */
		*authmethodp = LDAP_AUTH_SIMPLE;
		if ( binddn == NULL ) {
			*dnp = *pwp = NULL;
		} else {
			*dnp = strdup( binddn );
			*pwp = strdup( bindpasswd );
		}
	} else {
	    /* second time called: free the DN and password copies */
		if ( *dnp != NULL ) {
			free( *dnp );
			free( *pwp );
		}
	}

	return( LDAP_SUCCESS );
}

void
install_rebind_proc( LDAP *ld )
{
	ldap_set_rebind_proc( ld, get_rebind_credentials, NULL );
}


#ifdef TEST

#define HOST "belltower"

main( int argc, char **argv )
{
	int		i, rc;
	LDAP		*ld;

	if ( argc < 2 ) {
		fprintf( stderr, "%s: DN...\n", argv[ 0 ] );
		exit( 1 );
	}

	if ( (ld = ldap_init( HOST, LDAP_PORT )) == NULL ) {
		perror( HOST );
		exit( 1 );
	}

	install_rebind_proc( ld );

	binddn = "cn=Test";
	bindpasswd = "woof";

	for ( i = 1; i < argc; ++i ) {
		read_and_print( ld, argv[ i ] );
	}

	exit( 0 );
}


read_and_print( LDAP *ld, char *dn )
{
	int		rc;
	LDAPMessage	*res, *e;
	char		**vals;

	if (( rc = ldap_search_s( ld, dn, LDAP_SCOPE_BASE, "(objectClass=*)",
	    NULL, 0, &res )) == LDAP_SUCCESS ) {
		e = ldap_first_entry( ld, res );
		vals = ldap_get_values( ld, e, "cn" );
		printf( "Entry: %s\ncn: %s\n\n", ldap_get_dn( ld, e ),
			vals == NULL ? "(null)" : vals[ 0 ] );
		ldap_msgfree( res );
		putchar( '\n' );
	} else {
		ldap_perror( ld, dn );
	}
}
#endif /* TEST */
