#include <stdio.h>
#include <ldap.h>

main( int argc, char **argv )
{
	LDAP		*ld;
	LDAPMessage	*result;
	int		i;
	char		*url;

	if ( argc < 2 ) {
		fputs( "usage: urlsearch ldapurl...\n", stderr );
		exit( 1 );
	}

	if ( (ld = ldap_init( "ldap.netscape.com", LDAP_PORT )) == NULL ) {
		perror( "ldap_init" );
		exit( 1 );
	}

	for ( i = 1; i < argc; ++i ) {
		url = argv[ i ];
		result = NULL;
		printf( "Following LDAP URL %s...\n", url );

		if ( ldap_url_search_s( ld, url, 0, &result )
		    != LDAP_SUCCESS ) {
			ldap_perror( ld, url );
		}

		if ( result != NULL ) {
			print_entries_with_values( ld, result );
			ldap_msgfree( result );
		}
		putchar( '\n' );
	}

	ldap_unbind( ld );
	exit( 0 );
}

#ifdef TEST
print_entries_with_values( LDAP *ld, LDAPMessage *result ) 
{
	LDAPMessage	*e;
	BerElement	*ber;
	char		*dn, *attr;
	char		**vals;
	int		i;

	for ( e = ldap_first_entry( ld, result ); e != NULL;
	   e = ldap_next_entry(ld, e ) ) {
		if ( (dn = ldap_get_dn( ld, e )) != NULL ) {
			printf( "dn: %s\n", dn );
			ldap_memfree( dn );
		}
		for ( attr = ldap_first_attribute( ld, e, &ber );
		   attr != NULL;
		   attr = ldap_next_attribute( ld, e, ber )) {
			if ( (vals = ldap_get_values( ld, e, attr ))
			   != NULL ) {
				for ( i = 0; vals[i] != NULL; i++ ) {
					printf( "%s: %s\n", attr, vals[i] );
				}
				ldap_value_free( vals );
			}
			ldap_memfree( attr );
		}
		printf( "\n" );
		if ( ber != NULL ) {
			ber_free( ber, 0 );
		}
	}
}
#endif /* TEST */
