#include <stdio.h>
#include <ldap.h>

typedef void (CMD_FUNC)( LDAP *, int, char ** );
CMD_FUNC add, add_s, lbind, lbind_s, delete, delete_s, search, search_s;
void init( LDAP **, int, char ** );
int parse_command( char *, int *, char ** );
void help( void );
char *strtok_quote( char *, char * );

#define MAXARGS	20

main( int argc, char **argv )
{
	char	line[BUFSIZ];
	int	cargc;
	char	*cargv[MAXARGS];
	LDAP	*ld;

	ld = NULL;
	printf( "cmd: " );
	while ( fgets( line, sizeof(line), stdin ) != NULL ) {
		switch ( parse_command( line, &cargc, cargv ) ) {
		case 'a':	/* asynchronous add */
			add( ld, cargc, cargv );
			break;
		case 'A':	/* synchronous add */
			add_s( ld, cargc, cargv );
			break;
		case 'i':	/* init LDAP session */
			init( &ld, cargc, cargv );
			break;
		default:	/* invalid command */
			help();
			break;
		}
		printf( "cmd: " );
	}
}

int
parse_command( char *line, int *argcp, char **argv )
{
	char	*token;

	*argcp = 0;
	for ( token = strtok_quote( line, " \t" ); token != NULL;
	    token = strtok_quote( NULL, " \t" ) ) {
		if ( *argcp == MAXARGS ) {
			fprintf( stderr, "More than %d tokens\n", MAXARGS );
			exit( 1 );
		}
		argv[(*argcp)++] = token;
	}
	argv[*argcp] = NULL;

	return( line[0] );
}

char *
strtok_quote( char *line, char *sep )
{
	int		inquote;
	char		*tmp, *d;
	static char	*next;

	if ( line != NULL ) {
		next = line;
	}
	while ( *next && strchr( sep, *next ) ) {
		next++;
	}

	if ( *next == '\0' ) {
		next = NULL;
		return( NULL );
	}

	d = tmp = next;
	for ( inquote = 0; *next; next++ ) {
		switch ( *next ) {
		case '"':
			if ( inquote ) {
				inquote = 0;
			} else {
				inquote = 1;
			}
			break;

		default:
			if ( ! inquote ) {
				if ( strchr( sep, *next ) != NULL ) {
					*d++ = '\0';
					next++;
					return( tmp );
				}
			}
			*d++ = *next;
			break;
		}
	}
	*d = '\0';

	return( tmp );
}

void
help( void )
{
	printf( "Commands:\n" );
	printf( "\ta:\tAsynchronous add operation\n" );
	printf( "\tA:\tSynchronous add operation\n" );
	printf( "\ti:\tInitialize an LDAP session\n" );
}

void
add( LDAP *ld, int cargc, char **cargv )
{
	printf( "Add (async)\n" );
}

void
add_s( LDAP *ld, int cargc, char **cargv )
{
	printf( "Add (sync)\n" );
}

void
init( LDAP **ld, int cargc, char **cargv )
{
	printf( "Init\n" );
	if ( cargc != 3 ) {
		printf( "usage: i host port\n" );
		return;
	}
	if ( (*ld = ldap_init( cargv[1], atoi( cargv[2] ) )) == NULL ) {
		printf( "Could not initialize LDAP host %s port %d\n",
		    cargv[1], cargv[2] );
		return;
	}
}
